import os
import functools
import sys
import threading
from copy import deepcopy
from inspect import isfunction
from queue import Queue
from colour_printing.config import CPConfig
from colour_printing.exception import PrintMeError
from colour_printing.helper import DYE

stream = sys.stdout


def level_wrap(func):
    @functools.wraps(func)
    def wrap(self, *args, **kwargs):
        level = func.__name__.upper()
        default = self.cp._default.get(level, {})
        data = {}
        # 参数
        for i in self.cp._terms:
            try:
                data[i] = kwargs[i]
            except:
                dd = default.get(i, "")
                if isfunction(dd):
                    data[i] = dd()
                else:
                    data[i] = dd
        sep = kwargs.get('sep', " ")
        if args:
            data['message'] = sep.join([str(i) for i in args])
        # record
        self.handler_record(deepcopy(data))
        # 最高优先级,对data的操作会影响后续操作的输出内容
        res = func(self, data)
        # 日志
        if level not in self.log_filter and self.Master_switch:
            msg = self.cp._rawtemplate.format(**data) + "\n"
            self.queue.put(msg)
        # 打印
        if self.switch and self.Master_switch and level not in self.print_filter:
            end = kwargs.get('end', '\n')
            self.show(level, data=data, end=end)
        return res

    return wrap


class ColourPrinting(object):
    Master_switch = True
    DYE = DYE

    @classmethod
    def shutdown(cls):
        cls.Master_switch = False

    @level_wrap
    def info(self, data: dict):
        """最高优先级,对data的操作会影响后续操作(日志,打印)的输出内容"""

    @level_wrap
    def debug(self, data: dict):
        """最高优先级,对data的操作会影响后续操作(日志,打印)的输出内容"""

    @level_wrap
    def error(self, data: dict):
        """最高优先级,对data的操作会影响后续操作(日志,打印)的输出内容"""

    @level_wrap
    def warning(self, data: dict):
        """最高优先级,对data的操作会影响后续操作(日志,打印)的输出内容"""

    @level_wrap
    def success(self, data: dict):
        """最高优先级,对data的操作会影响后续操作(日志,打印)的输出内容"""

    def handler_record(self, record: dict):
        """record不受level函数影响,处理日志信息 重写此函数以应用每个不同的使用场景 """


class LogHandler(object):
    def __init__(self, printme):
        self.printme = printme
        self.log_path = os.getcwd()
        self.log_name = 'colour_printing_log.log'
        self.main_t = None

    def run(self, log_path: str = '', log_name: str = ''):  # log
        """
        :param log_path: 路径
        :param log_name: 文件名
        :return:
        """
        if log_path:
            self.log_path = log_path
        if log_name:
            self.log_name = log_name
        self.main_t = threading.currentThread()
        t = threading.Thread(target=self.__log_to_file, args=())
        t.start()

    def __log_to_file(self):
        path = os.path.join(self.log_path, self.log_name)
        stream.write('[*]Tip>> 日志文件path: {path}\n'.format(path=path))
        with open(path, 'a+') as f:
            while True:
                if self.printme.queue.empty():
                    if self.main_t.isAlive():
                        continue
                    else:
                        break
                f.write(self.printme.queue.get())
                f.flush()


class PrintMe(ColourPrinting):

    def __init__(self, cp: CPConfig, **kwargs):
        if not isinstance(cp, CPConfig):
            raise PrintMeError("type must be CPconfig")
        # style config
        self.cp = cp
        # switch
        self.__switch = True
        self.__print_filter = []
        self.__log_filter = []
        # log
        self.queue = Queue()
        self.log_handler = LogHandler(printme=self)
        # custom args
        for k, v in kwargs.items():
            if k in dir(self):
                raise PrintMeError('变量名"{k}"已被定义咯,请更换其他变量名'.format(k=k))
            setattr(self, k, v)

    def show(self, level: str, data: dict, end: str):
        # style
        if not self.DYE or not self.cp._box.get(level.upper()):
            msg = self.cp._rawtemplate.format(**data)
        else:
            style = self.cp._box.get(level.upper())
            data.update(style)
            msg = self.cp._template.format(**data)
        stream.write(msg)
        stream.write(end)

    def set_default(self, **kwargs):
        """
        设置默认值
        :param kwargs:
        :return:
        """
        level = kwargs.get('set_level')
        if level:
            if level.upper() in self.cp._levels:
                for k, v in kwargs.items():
                    self.cp._default.setdefault(level.upper(), {}).update({k: v})
        else:
            for level in self.cp._levels:
                for k, v in kwargs.items():
                    self.cp._default.setdefault(level.upper(), {}).update({k: v})

    @property
    def switch(self):
        return self.__switch

    def close(self):
        self.__switch = False

    @property
    def print_filter(self):
        return self.__print_filter

    @print_filter.setter
    def print_filter(self, val: list):
        self.__print_filter = [i.upper() for i in val]

    @property
    def log_filter(self):
        return self.__log_filter

    @log_filter.setter
    def log_filter(self, val: list):
        self.__log_filter = [i.upper() for i in val]



import re

from colour_printing.exception import PrintMeError
from colour_printing.helper import check
from colour_printing.style import setting, CPStyle


class Term(object):
    def __init__(self, *args, default=''):
        self.default = default
        data = {}
        for i in args:
            if isinstance(i, CPStyle):
                data[i.name] = i.value
        self.style = setting(**data)  # list


class CPConfig(object):
    def __init__(self, template):
        self._box = {}
        self._default = {}
        self._levels = []
        self._check(template)

    def _check(self, template):
        self._rawtemplate = template
        self._terms = re.findall(r'(?<=\{)[^}]*(?=\})+', template)
        e = check(self._terms)
        if e:
            raise PrintMeError(e)
        term_wrap = {}
        for t in self._terms:
            term_wrap[t] = "{%s}{%s}{%s}" % (t + '0', t, t + '1')
        self._template = self._rawtemplate.format(**term_wrap)

    def wrap(self, func):
        func(self)
        self.__fill(func.__name__.upper())

    def __fill(self, level_name):
        self._levels.append(level_name)
        for t in self._terms:
            term = getattr(self, t, Term())
            self._box.setdefault(level_name, {}).update(
                {"{t}0".format(t=t): term.style[0], "{t}1".format(t=t): term.style[1]})
            self._default.setdefault(level_name, {}).update({t: term.default})
            if hasattr(self, t):
                delattr(self, t)

    def set_all_default(self, **kwargs):
        """
        设置默认值
        :param kwargs:
        :return:
        """
        for level in self._levels:
            for k, v in kwargs.items():
                self._default.setdefault(level.upper(), {}).update({k: v})



from colour_printing.default import log
from colour_printing.custom import PrintMe

PrintMe.DYE = False
# log.info('hello')
# log.success('hello', flag='', level='ss')
# log.error('hello')
# log.warning('hello')
# log.debug('hello')
from colour_printing import cprint, Fore

cprint('sd', Fore.BLUE)



"""
#                     *Colour-printing Reference*
#########################################################################################
#   @'fore': # 前景色         @'back':# 背景              @'mode':# 显示模式               # 
#            'black': 黑色            'black':  黑色              'normal': 终端默认设置   # 
#            'red': 红色              'red':  红色                'bold':  高亮显示        # 
#            'green': 绿色            'green': 绿色               'underline':  使用下划线 #
#            'yellow': 黄色           'yellow': 黄色              'blink': 闪烁           # 
#            'blue':  蓝色            'blue':  蓝色               'invert': 反白显示       #    
#            'purple':  紫红色        'purple':  紫红色            'hide': 不可见          #    
#            'cyan':  青蓝色          'cyan':  青蓝色                                     #
#            'white':  白色           'white':  白色                                     #
#########################################################################################
"""

from colour_printing.config import CPConfig, Term

from datetime import datetime
from colour_printing import Mode, Fore, Back

get_time = lambda: datetime.strftime(datetime.now(), "%Y-%m-%d %H:%M:%S.%f")[:-3]
fill = 7

TEMPLATE = "{time} {flag} {message}"
CP = CPConfig(TEMPLATE)


@CP.wrap
def info(cp):
    cp.time = Term(Fore.CYAN)
    cp.flag = Term(Fore.BLUE, Mode.INVERT, default="INFO".center(fill, "-"))
    cp.message = Term(Fore.BLUE)


@CP.wrap
def error(cp):
    cp.time = Term(Fore.CYAN)
    cp.flag = Term(Fore.RED, Mode.INVERT, default="ERROR".center(fill, "-"))
    cp.message = Term(Fore.RED)


@CP.wrap
def success(cp):
    cp.time = Term(Fore.CYAN)
    cp.flag = Term(Fore.GREEN, Mode.INVERT, default="SUCCESS".center(fill, "-"))
    cp.message = Term(Fore.GREEN)


@CP.wrap
def debug(cp):
    cp.time = Term(Fore.CYAN)
    cp.flag = Term(Fore.PURPLE, Mode.INVERT, default="DEBUG".center(fill, "-"))
    cp.message = Term(Fore.PURPLE)


@CP.wrap
def warning(cp):
    cp.time = Term(Fore.CYAN)
    cp.flag = Term(Fore.YELLOW, Mode.INVERT, default="WARNING".center(fill, "-"))
    cp.message = Term(Fore.YELLOW)


CP.set_all_default(time=get_time)



import re
import sys
import argparse
import os
from colour_printing.helper import check

stream = sys.stdout
parser = argparse.ArgumentParser()
parser.description = "***** Colour-printing ***** github@Faithforus "
parser.add_argument('-n', '--filename', default="colour_printing_config.py", help='配置文件名.py')
parser.add_argument('-t', '--template', help="信息模板,例: {}{}{message}")
parser.add_argument('-l', '--newLevel', help="新增level,配合@level_wrap使用,多个用分号隔开")


def tip(msg):
    stream.write("[*]Tip>> {msg}".format(msg=msg))
    stream.write('\n')


header = """\"""
#                     *Colour-printing Reference*
#########################################################################################
#   @'fore': # 前景色         @'back':# 背景              @'mode':# 显示模式               # 
#            'black': 黑色            'black':  黑色              'normal': 终端默认设置   # 
#            'red': 红色              'red':  红色                'bold':  高亮显示        # 
#            'green': 绿色            'green': 绿色               'underline':  使用下划线 #
#            'yellow': 黄色           'yellow': 黄色              'blink': 闪烁           # 
#            'blue':  蓝色            'blue':  蓝色               'invert': 反白显示       #    
#            'purple':  紫红色        'purple':  紫红色            'hide': 不可见          #    
#            'cyan':  青蓝色          'cyan':  青蓝色                                     #
#            'white':  白色           'white':  白色                                     #
#########################################################################################
\"""\n\n"""

lk = '{'
rk = '}'
dy = "\"\""
tab = " " * 4


def lib_template(template):
    return """
from colour_printing.config import CPConfig, Term\n
from datetime import datetime\n
from colour_printing import Mode, Fore, Back\n
get_time = lambda: datetime.strftime(datetime.now(), '%Y-%m-%d %H:%M:%S.%f')[:-3]\n
TEMPLATE = "{template}"
CP = CPConfig(TEMPLATE)  # 从其他地方导入我""".format(template=template)


def cls_template(level_list, terms):
    result = ""
    for level in level_list:
        result += """\n\n
@CP.wrap
def {level}(cp):""".format(level=level)
        for term in terms:
            result += """
{tab}cp.{term} = Term()""".format(tab=tab, term=term)
    return result


def assemble(level_list, terms, template):
    result = header
    result += lib_template(template)
    result += cls_template(level_list, terms)
    return result


def create_py_file(file_path, level_list, terms, template):
    ns = assemble(level_list, terms, template)
    with open(file_path, 'w')as f:
        f.write(ns)
    tip(f'创建配置模板文件完成-->  {file_path}')
    return 0


def template_handle(template):
    # template
    terms = re.findall(r'(?<=\{)[^}]*(?=\})+', template)
    e = check(terms)
    if e:
        tip(e)
        sys.exit(2)
    return terms


def execute():
    if len(sys.argv) <= 1:
        tip("cprint -h 查看帮助")
        sys.exit(0)
    args = parser.parse_args()
    template = args.template
    name = args.filename
    new_level = args.newLevel
    # handle
    terms = template_handle(template)
    # filepath
    name = name if name.endswith('.py') else name + '.py'
    file_path = os.getcwd() + '/' + name
    if os.path.exists(file_path):
        tip("{name}该配置文件已存在,确认要覆写吗?将配置丢失".format(name=name))
        if input("[Y/n]:") != "Y":
            tip('cancel!')
            sys.exit(0)
    # level
    level_list = ['info', 'success', 'warning', 'error', 'debug']
    if new_level:
        level_list += [l.strip() for l in new_level.split(";") if l.strip()]
    code = create_py_file(file_path, level_list, terms, template)
    sys.exit(code)


if __name__ == '__main__':
    execute()



STYLE = {
    'fore':
        {  # 前景色
            'black': 30,  # 黑色
            'red': 31,  # 红色
            'green': 32,  # 绿色
            'yellow': 33,  # 黄色
            'blue': 34,  # 蓝色
            'purple': 35,  # 紫红色
            'cyan': 36,  # 青蓝色
            'white': 37,  # 白色
        },

    'back':
        {  # 背景
            'black': 40,  # 黑色
            'red': 41,  # 红色
            'green': 42,  # 绿色
            'yellow': 43,  # 黄色
            'blue': 44,  # 蓝色
            'purple': 45,  # 紫红色
            'cyan': 46,  # 青蓝色
            'white': 47,  # 白色
        },

    'mode':
        {  # 显示模式
            'normal': 0,  # 终端默认设置
            'bold': 1,  # 高亮显示
            'underline': 4,  # 使用下划线
            'blink': 5,  # 闪烁
            'invert': 7,  # 反白显示
            'hide': 8,  # 不可见
        },

    'default':
        {
            'end': 0,
        },
}


def setting(mode='', fore='', back=''):
    """
    linxu:转义序列以ESC开头，即ASCII码下的\033
               格式: \033[显示方式;前景色;背景色m
    """
    mode = '%s' % STYLE['mode'][mode] if STYLE['mode'].get(mode) else ''

    fore = '%s' % STYLE['fore'][fore] if STYLE['fore'].get(fore) else ''

    back = '%s' % STYLE['back'][back] if STYLE['back'].get(back) else ''

    style = ';'.join([s for s in [mode, fore, back] if s])

    style = '\033[%sm' % style if style else ''

    end = '\033[%sm' % STYLE['default']['end'] if style else ''

    return style, end


class CPStyle(object):
    def __init__(self, name, value):
        self.name = name
        self.value = value


class Fore:
    BLACK = CPStyle(name="fore", value="black")
    RED = CPStyle(name="fore", value="red")
    GREEN = CPStyle(name="fore", value="green")
    YELLOW = CPStyle(name="fore", value="yellow")
    BLUE = CPStyle(name="fore", value="blue")
    PURPLE = CPStyle(name="fore", value="purple")
    CYAN = CPStyle(name="fore", value="cyan")
    WHITE = CPStyle(name="fore", value="white")


class Back:
    BLACK = CPStyle(name="back", value="black")
    RED = CPStyle(name="back", value="red")
    GREEN = CPStyle(name="back", value="green")
    YELLOW = CPStyle(name="back", value="yellow")
    BLUE = CPStyle(name="back", value="blue")
    PURPLE = CPStyle(name="back", value="purple")
    CYAN = CPStyle(name="back", value="cyan")
    WHITE = CPStyle(name="back", value="white")


class Mode:
    NORMAL = CPStyle(name="mode", value="normal")
    BOLD = CPStyle(name="mode", value="bold")
    UNDERLINE = CPStyle(name="mode", value="underline")
    BLINK = CPStyle(name="mode", value="blink")
    INVERT = CPStyle(name="mode", value="invert")
    HIDE = CPStyle(name="mode", value="hide")



from colour_printing import cprint, cword, Fore

# 打印色彩字符
cprint('default')
cprint('hello', Fore.RED)

# 或者只要色彩字符
s1 = cword('I', Fore.YELLOW)
s2 = cword('LOVE', 'China', Fore.RED)

print(s1, s2[0], s2[1])


import platform

if platform.system() == 'Linux':
    DYE = True
else:
    DYE = False


def check(terms):
    if "message" not in terms:
        return 'TEMPLATE中未找到 {message} ! '
    for t in terms:
        if t.strip() == '':
            return '未知 {} ! '
        if " " in t:
            return '{{t}} 含空格'.format(t)
    return None



import setuptools

try:
    import pypandoc

    long_description = pypandoc.convert_file('README.md', 'rst')
except Exception:
    long_description = ""

setuptools.setup(
    name="colour-printing",
    version="0.3.20",
    author="faithforus",
    author_email="ljunf817@163.com",
    description="colour-printing",
    long_description=long_description,
    keywords="python package print",
    url="https://github.com/Faithforus/Colour-printing",
    license="MIT",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    entry_points={
        'console_scripts': ['cprint = colour_printing.cmdline:execute']
    },
)



from colour_printing.style import setting, CPStyle


def cprint(*args, **kwargs):
    """
    :param args: 字符
    :param kwargs: 颜色参数
    :return: list 彩色字符
    sep=' ', end='\n', file=None
    """
    sep = kwargs.pop('sep', " ")
    end = kwargs.pop('end', '\n')
    file = kwargs.pop('file', None)
    str_temp = cword(*args)
    if not isinstance(str_temp, list):
        str_temp = [str_temp]
    if file:
        temp = []
        for i in args:
            if not isinstance(i, CPStyle):
                temp.append(i)
        print(*temp, sep=sep, end=end, file=file)
    else:
        print(*str_temp, sep=sep, end=end)


def cword(*args):
    """
    :param args: 字符
    :param kwargs: 颜色参数
    :return: list 彩色字符
    """
    data = {}
    temp = []
    for i in args:
        if isinstance(i, CPStyle):
            data[i.name] = i.value
        else:
            temp.append(i)
    style = setting(**data)
    str_temp = []
    for s in temp:
        str_temp.append('{s0}{s}{s1}'.format(s0=style[0], s=s, s1=style[1]))
    return str_temp[0] if len(str_temp) == 1 else str_temp



# import errno
# import os
# import sys
# import types
#
# stream = sys.stdout
#
#
# def import_string(import_name, silent=False):
#     import_name = str(import_name).replace(":", ".")
#     try:
#         try:
#             __import__(import_name)
#         except ImportError:
#             if "." not in import_name:
#                 raise
#         else:
#             return sys.modules[import_name]
#
#         module_name, obj_name = import_name.rsplit(".", 1)
#         module = __import__(module_name, globals(), locals(), [obj_name])
#         try:
#             return getattr(module, obj_name)
#         except AttributeError as e:
#             raise ImportError(e)
#
#     except ImportError as e:
#         raise e
#
#
# string_types = (str,)
# integer_types = (int,)
#
# """从.py文件导入"""
#
#
# class Config(dict):
#     def __init__(self):
#         dict.__init__(self)
#         self.config_str = ""
#         self.filename = ""
#
#     def from_pyfile(self, file_path, silent=False):
#         filename = os.path.split(file_path)[1]
#         self.filename = filename
#         d = types.ModuleType('config')
#         d.__file__ = filename
#         try:
#             with open(file_path, mode='rb') as config_file:
#                 config = config_file.read()
#                 self.config_str = config.decode()
#                 exec(compile(config, filename, 'exec'), d.__dict__)
#         except IOError as e:
#             if silent and e.errno in (
#                     errno.ENOENT, errno.EISDIR, errno.ENOTDIR
#             ):
#                 return False
#             e.strerror = 'Unable to load configuration file (%s)' % e.strerror
#             raise
#         return self.from_object(d)
#
#     def from_object(self, obj):
#         if isinstance(obj, string_types):
#             obj = import_string(obj)
#         for key in dir(obj):
#             if key.isupper():
#                 self[key] = getattr(obj, key)
#         return True
